@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# This file is part of restkit released under the MIT license. 
+# This file is part of restkit released under the MIT license.
 # See the NOTICE for more information.
 import base64
 import errno
@@ -19,7 +19,7 @@ try:
 except ImportError:
     if hasattr(socket, "ssl"):
         from httplib import FakeSocket
-        from .sock import trust_all_certificates
+        from restkit.sock import trust_all_certificates
 
         @trust_all_certificates
         def _ssl_wrapper(sck, **kwargs):
@@ -29,18 +29,24 @@ except ImportError:
     else:
         have_ssl = False
 
-from . import __version__
-from .conn import Connection
-from .errors import RequestError, RequestTimeout, RedirectLimit, \
+try:
+    from http_parser.http import HttpStream
+    from http_parser.reader import SocketReader
+except ImportError:
+    raise ImportError("""http-parser isn't installed.
+        
+        pip install http-parser""")
+
+from restkit import __version__
+
+from restkit.conn import Connection
+from restkit.errors import RequestError, RequestTimeout, RedirectLimit, \
 NoMoreData, ProxyError
-from .globals import get_manager 
-from . import http
-
-from .sock import close, send, sendfile, sendlines, send_chunk, \
+from restkit.globals import get_manager
+from restkit.sock import close, send, sendfile, sendlines, send_chunk, \
 validate_ssl_args
-from .util import parse_netloc, rewrite_location
-from .wrappers import Request, Response
-
+from restkit.util import parse_netloc, rewrite_location
+from restkit.wrappers import Request, Response
 
 MAX_CLIENT_TIMEOUT=300
 MAX_CLIENT_CONNECTIONS = 5
@@ -55,8 +61,8 @@ class Client(object):
 
     """ A client handle a connection at a time. A client is threadsafe,
     but an handled shouldn't be shared between threads. All connections
-    are shared between threads via a pool. 
-    
+    are shared between threads via a pool.
+
     >>> from restkit import *
     >>> c = Client()
     >>> r = c.request("http://google.com")
@@ -68,7 +74,7 @@ class Client(object):
     >>> r = c.request("http://google.com")
     >>> r.status
     '200 OK'
-     
+
     """
 
     version = (1, 1)
@@ -78,7 +84,7 @@ class Client(object):
             follow_redirect=False,
             force_follow_redirect=False,
             max_follow_redirect=MAX_FOLLOW_REDIRECTS,
-            filters=None, 
+            filters=None,
             decompress=True,
             max_status_line_garbage=None,
             max_header_count=0,
@@ -114,7 +120,7 @@ class Client(object):
         :parama response_class: the response class to use
         :param timeout: the default timeout of the connection
         (SO_TIMEOUT)
-        
+
         :param max_tries: the number of tries before we give up a
         connection
         :param wait_tries: number of time we wait between each tries.
@@ -123,7 +129,7 @@ class Client(object):
         """
         self.follow_redirect = follow_redirect
         self.force_follow_redirect = force_follow_redirect
-        self.max_follow_redirect = max_follow_redirect  
+        self.max_follow_redirect = max_follow_redirect
         self.decompress = decompress
         self.filters = filters or []
         self.max_status_line_garbage = max_status_line_garbage
@@ -133,14 +139,14 @@ class Client(object):
         self.request_filters = []
         self.response_filters = []
         self.load_filters()
-        
-                
+
+
         # set manager
         if manager is None:
             manager = get_manager()
         self._manager = manager
 
-        # change default response class 
+        # change default response class
         if response_class is not None:
             self.response_class = response_class
 
@@ -152,7 +158,7 @@ class Client(object):
         self._url = None
         self._initial_url = None
         self._write_cb = None
-        self._headers = None 
+        self._headers = None
         self._sock_key = None
         self._sock = None
         self._original = None
@@ -160,7 +166,7 @@ class Client(object):
         self.method = 'GET'
         self.body = None
         self.ssl_args = ssl_args or {}
-        
+
     def load_filters(self):
         """ Populate filters from self.filters.
         Must be called each time self.filters is updated.
@@ -175,18 +181,18 @@ class Client(object):
         """ create a socket """
         if log.isEnabledFor(logging.DEBUG):
             log.debug("create new connection")
-        for res in socket.getaddrinfo(addr[0], addr[1], 0, 
+        for res in socket.getaddrinfo(addr[0], addr[1], 0,
                 socket.SOCK_STREAM):
             af, socktype, proto, canonname, sa = res
 
             try:
                 sck = socket.socket(af, socktype, proto)
-                
+
                 if self.timeout is not None:
                     sck.settimeout(self.timeout)
 
                 sck.connect(sa)
-                    
+
                 if is_ssl:
                     if not have_ssl:
                         raise ValueError("https isn't supported.  On python 2.5x,"
@@ -195,10 +201,13 @@ class Client(object):
                                         + "to be intalled.")
                     validate_ssl_args(self.ssl_args)
                     sck = _ssl_wrapper(sck, **self.ssl_args)
-                
+
                 return sck
-            except socket.error:
-                close(sck)
+            except socket.error, ex:
+                if ex == 24: # too many open files
+                    raise
+                else:
+                    close(sck)
         raise socket.error, "Can't connect to %s" % str(addr)
 
     def get_connection(self, request):
@@ -210,7 +219,8 @@ class Client(object):
         extra_headers = []
         sck = None
         if self.use_proxy:
-            sck, addr, extra_headers = self.proxy_connection(request, addr, ssl)
+            sck, addr, extra_headers = self.proxy_connection(request,
+                    addr, is_ssl)
         if not sck:
             sck = self._manager.find_socket(addr, is_ssl)
             if sck is None:
@@ -222,18 +232,20 @@ class Client(object):
 
         connection = Connection(sck, self._manager, addr,
                 ssl=is_ssl, extra_headers=extra_headers)
-        return connection 
+        return connection
 
-    def proxy_connection(self, request, req_addr, ssl):
+    def proxy_connection(self, request, req_addr, is_ssl):
         """ do the proxy connection """
         proxy_settings = os.environ.get('%s_proxy' %
                 request.parsed_url.scheme)
 
         if proxy_settings and proxy_settings is not None:
+            request.is_proxied = True
+
             proxy_settings, proxy_auth =  _get_proxy_auth(proxy_settings)
             addr = parse_netloc(urlparse.urlparse(proxy_settings))
 
-            if ssl:
+            if is_ssl:
                 if proxy_auth:
                     proxy_auth = 'Proxy-authorization: %s' % proxy_auth
                 proxy_connect = 'CONNECT %s:%s HTTP/1.0\r\n' % req_addr
@@ -242,12 +254,12 @@ class Client(object):
                 if not user_agent:
                     user_agent = "User-Agent: restkit/%s\r\n" % __version__
 
-                proxy_pieces = '%s%s%s\r\n' % (proxy_connect, proxy_auth, 
+                proxy_pieces = '%s%s%s\r\n' % (proxy_connect, proxy_auth,
                         user_agent)
 
-                sck = self._manager.find_socket(addr, ssl)
+                sck = self._manager.find_socket(addr, is_ssl)
                 if sck is None:
-                    self = self.connect(addr, ssl)
+                    sck = self.connect(addr, is_ssl)
 
                 send(sck, proxy_pieces)
                 unreader = http.Unreader(sck)
@@ -263,9 +275,9 @@ class Client(object):
                 if proxy_auth:
                     headers = [('Proxy-authorization', proxy_auth)]
 
-                sck = self._manager.find_socket(addr, ssl)
+                sck = self._manager.find_socket(addr, is_ssl)
                 if sck is None:
-                    sck = self.connect(addr, ssl)
+                    sck = self.connect(addr, is_ssl)
                 return sck, addr, headers
         return None, req_addr, []
 
@@ -286,15 +298,20 @@ class Client(object):
 
         ua = headers.iget('user_agent')
         if not ua:
-            ua = USER_AGENT 
+            ua = USER_AGENT
         host = request.host
 
         accept_encoding = headers.iget('accept-encoding')
         if not accept_encoding:
             accept_encoding = 'identity'
 
+        if request.is_proxied:
+            full_path = ("https://" if request.is_ssl() else "http://") + request.host + request.path
+        else:
+            full_path = request.path
+
         lheaders = [
-            "%s %s %s\r\n" % (request.method, request.path, httpver),
+            "%s %s %s\r\n" % (request.method, full_path, httpver),
             "Host: %s\r\n" % host,
             "User-Agent: %s\r\n" % ua,
             "Accept-Encoding: %s\r\n" % accept_encoding
@@ -306,7 +323,7 @@ class Client(object):
         if log.isEnabledFor(logging.DEBUG):
             log.debug("Send headers: %s" % lheaders)
         return "%s\r\n" % "".join(lheaders)
-        
+
     def perform(self, request):
         """ perform the request. If an error happen it will first try to
         restart it """
@@ -314,7 +331,7 @@ class Client(object):
         if log.isEnabledFor(logging.DEBUG):
             log.debug("Start to perform request: %s %s %s" %
                     (request.host, request.method, request.path))
-                
+
         tries = self.max_tries
         wait = self.wait_tries
         while tries > 0:
@@ -326,7 +343,7 @@ class Client(object):
                 # send headers
                 msg = self.make_headers_string(request,
                         connection.extra_headers)
-                
+
                 # send body
                 if request.body is not None:
                     chunked = request.is_chunked()
@@ -375,7 +392,7 @@ class Client(object):
                         send_chunk(sck, "")
                 else:
                     sck.sendall(msg)
-                
+
                 return self.get_response(request, connection)
             except socket.gaierror, e:
                 try:
@@ -400,12 +417,10 @@ class Client(object):
                 except:
                     pass
 
-                if e[0] not in (errno.EAGAIN, errno.ECONNABORTED, 
-                        errno.EPIPE, errno.ECONNREFUSED, 
+                if e[0] not in (errno.EAGAIN, errno.ECONNABORTED,
+                        errno.EPIPE, errno.ECONNREFUSED,
                         errno.ECONNRESET, errno.EBADF) or tries <= 0:
-                    raise RequestError("socker.error: %s" % str(e))
-            except (KeyboardInterrupt, SystemExit):
-                break
+                    raise RequestError("socket.error: %s" % str(e))
             except (StopIteration, NoMoreData):
                 connection.close()
                 if tries <= 0:
@@ -416,12 +431,12 @@ class Client(object):
                                 not isinstance(request.body, types.StringTypes):
                             raise RequestError("connection closed and can't"
                                     + "be resent")
-	    except:
+            except Exception:
                 # unkown error
                 log.debug("unhandled exception %s" %
                         traceback.format_exc())
                 raise
-            
+
             # time until we retry.
             time.sleep(wait)
             wait = wait * 2
@@ -433,7 +448,7 @@ class Client(object):
 
         request = Request(url, method=method, body=body,
                 headers=headers)
-        
+
         # apply request filters
         # They are applied only once time.
         for f in self.request_filters:
@@ -455,15 +470,9 @@ class Client(object):
         if request.initial_url is None:
             request.initial_url = self.url
 
-        # discard response body and reset request informations
-        if hasattr(resp, "_body"):
-            resp._body.discard()
-        else:
-            resp.body.discard()
-
         # make sure location follow rfc2616
         location = rewrite_location(request.url, location)
-       
+
         if log.isEnabledFor(logging.DEBUG):
             log.debug("Redirect to %s" % location)
 
@@ -481,23 +490,17 @@ class Client(object):
         if log.isEnabledFor(logging.DEBUG):
             log.debug("Start to parse response")
 
-        unreader = http.Unreader(connection.socket())
-        while True:
-            resp = http.Request(unreader, decompress=self.decompress,
-                    max_status_line_garbage=self.max_status_line_garbage,
-                    max_header_count=self.max_header_count)
-            if resp.status_int != 100:
-                break
-            resp.body.discard()
+        p = HttpStream(SocketReader(connection.socket()), kind=1, 
+                decompress=self.decompress)
 
         if log.isEnabledFor(logging.DEBUG):
-            log.debug("Got response: %s" % resp.status)
-            log.debug("headers: [%s]" % resp.headers)
-        
-        location = resp.headers.iget('location')
+            log.debug("Got response: %s" % p.status())
+            log.debug("headers: [%s]" % p.headers())
 
+        location = p.headers().get('location')
+        
         if self.follow_redirect:
-            if resp.status_int in (301, 302, 307,):
+            if p.status_code() in (301, 302, 307,):
                 if request.method in ('GET', 'HEAD',) or \
                         self.force_follow_redirect:
                     if hasattr(self.body, 'read'):
@@ -509,16 +512,16 @@ class Client(object):
                                     "because body has already been read"
                                     % (self.url, location))
                     connection.release()
-                    return self.redirect(resp, location, request)
+                    return self.redirect(p, location, request)
 
-            elif resp.status_int == 303 and self.method == "POST":
+            elif p.status_code() == 303 and self.method == "POST":
                 connection.release()
                 request.method = "GET"
                 request.body = None
-                return self.redirect(resp, location, request)
+                return self.redirect(p, location, request)
 
         # create response object
-        resp = self.response_class(connection, request, resp)
+        resp = self.response_class(connection, request, p)
 
         # apply response filters
         for f in self.response_filters:
@@ -544,11 +547,11 @@ def _get_proxy_auth(proxy_settings):
     if not proxy_username:
         u = urlparse.urlparse(proxy_settings)
         if u.username:
-            proxy_password = u.password or proxy_password 
-            proxy_settings = urlparse.urlunparse((u.scheme, 
-                u.netloc.split("@")[-1], u.path, u.params, u.query, 
+            proxy_password = u.password or proxy_password
+            proxy_settings = urlparse.urlunparse((u.scheme,
+                u.netloc.split("@")[-1], u.path, u.params, u.query,
                 u.fragment))
-    
+
     if proxy_username:
         user_auth = base64.encodestring('%s:%s' % (proxy_username,
                                     proxy_password))
