@@ -5,7 +5,7 @@
 
 import logging
 
-from . import sock
+from restkit import sock
 
 log = logging.getLogger(__name__)
 
@@ -18,9 +18,12 @@ class Connection(object):
         self.addr = addr
         self.ssl = ssl
         self.extra_headers = extra_headers
-        
+        self._released = False
 
     def release(self, should_close=False):
+        if self._released:
+            return
+
         if should_close:
             self.close() 
         else:
@@ -28,6 +31,8 @@ class Connection(object):
                 log.debug("release connection")
             self.manager.store_socket(self._sock, self.addr, self.ssl)
 
+        self._released = True
+
     def close(self):
         if log.isEnabledFor(logging.DEBUG):
             log.debug("close connection")
