@@ -3,7 +3,7 @@
 # This file is part of restkit released under the MIT license. 
 # See the NOTICE for more information.
 
-from .manager import Manager
+from restkit.manager import Manager
 
 _manager = None
 
