@@ -3,4 +3,4 @@
 # This file is part of restkit released under the MIT license. 
 # See the NOTICE for more information.
 
-from .base import Manager
+from restkit.manager.base import Manager
