@@ -18,9 +18,7 @@ except ImportError:
     from StringIO import StringIO
 import tempfile
 
-
-from .http import LengthReader
-from . import sock
+from restkit import sock
 
 class TeeInput(object):
     
@@ -185,15 +183,14 @@ class ResponseTeeInput(TeeInput):
     def __init__(self, resp, connection, should_close=False):
         self.buf = StringIO()
         self.resp = resp
-        self.stream =resp._body
+        self.stream =resp.body_stream()
         self.connection = connection
         self.should_close = should_close
         self.eof = False
         
         # set temporary body
-        if isinstance(resp._body.reader, LengthReader):
-            clen = int(resp.headers.iget('content-length'))
-            
+        clen = int(resp.headers.get('content-length') or -1)
+        if clen >= 0:
             if (clen <= sock.MAX_BODY):
                 self.tmp = StringIO()
             else:
@@ -209,7 +206,7 @@ class ResponseTeeInput(TeeInput):
          
     def _close_unreader(self):
         if not self.eof:
-            self.resp._body.discard()
+            self.stream.close()
         self.connection.release(self.should_close)
             
     def _finalize(self):
